@@ -10,14 +10,19 @@
 #' dim_desc(1:10)
 #' dim_desc(Titanic)
 dim_desc <- function(x) {
-  dim <- dim(x) %||% length(x)
-  format_dim <- map_chr(dim, big_mark)
+  dim <- dim2(x)
+  format_dim <- big_mark(dim)
   format_dim[is.na(dim)] <- "??"
   paste0(format_dim, collapse = spaces_around(mult_sign()))
 }
 
+# https://github.com/r-lib/vctrs/pull/467
+dim2 <- function(x) {
+  dim(x) %||% vctrs::vec_size(x)
+}
+
 size_sum <- function(x) {
-  if (!is_vector_s3(x)) return("")
+  if (!vctrs::vec_is(x)) return("")
 
   paste0(" [", dim_desc(x), "]")
 }
